'use strict';

var React = require('react');
var useResponsiveValue = require('../hooks/useResponsiveValue.js');
var octiconsReact = require('@primer/octicons-react');
var getBreakpointDeclarations = require('../utils/getBreakpointDeclarations.js');
var warning = require('../utils/warning.js');
var clsx = require('clsx');
var PageHeader_module = require('./PageHeader.module.css.js');
var toggleSxComponent = require('../internal/utils/toggleSxComponent.js');
var defaultSxProp = require('../utils/defaultSxProp.js');
var jsxRuntime = require('react/jsx-runtime');
var useProvidedRefOrCreate = require('../hooks/useProvidedRefOrCreate.js');
var Heading = require('../Heading/Heading.js');
var Link = require('../Link/Link.js');
var Box = require('../Box/Box.js');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var React__default = /*#__PURE__*/_interopDefault(React);

// Default state for the `visible` prop when a sub component is only visible on narrow viewport
const hiddenOnRegularAndWide = {
  narrow: false,
  regular: true,
  wide: true
};

// Default state for the `visible` prop when a sub component is visible on regular and wide viewport
const hiddenOnNarrow = {
  narrow: true,
  regular: false,
  wide: false
};

// Root
// -----------------------------------------------------------------------------

const RootBaseComponent = toggleSxComponent.toggleSxComponent('div');
const Root = /*#__PURE__*/React__default.default.forwardRef(({
  children,
  className,
  sx = defaultSxProp.defaultSxProp,
  as = 'div',
  'aria-label': ariaLabel,
  role,
  hasBorder
}, forwardedRef) => {
  const rootRef = useProvidedRefOrCreate.useProvidedRefOrCreate(forwardedRef);
  const isInteractive = element => {
    return ['a', 'button'].some(selector => element.matches(selector)) || element.hasAttribute('role') && element.getAttribute('role') === 'button' || element.hasAttribute('link') && element.getAttribute('role') === 'link' || element.hasAttribute('tabindex');
  };
  React.useEffect(function validateInteractiveElementsInTitle() {
    if (!(process.env.NODE_ENV !== "production")) return;
    let hasContextArea = false;
    let hasLeadingAction = false;
    if (!rootRef.current || rootRef.current.children.length <= 0) return;
    const titleArea = Array.from(rootRef.current.children).find(child => {
      return child instanceof HTMLElement && child.getAttribute('data-component') === 'TitleArea';
    });

    // It is very unlikely to have a PageHeader without a TitleArea, but we still want to make sure we don't break the page if that happens.
    if (!titleArea) return;
    for (const child of React__default.default.Children.toArray(children)) {
      if (/*#__PURE__*/React__default.default.isValidElement(child) && child.type === ContextArea) {
        hasContextArea = true;
      }
      if (/*#__PURE__*/React__default.default.isValidElement(child) && child.type === LeadingAction) {
        hasLeadingAction = true;
      }
    }
    // Check if TitleArea has any interactive children or grandchildren.
    const hasInteractiveContent = Array.from(titleArea.childNodes).some(child => {
      return child instanceof HTMLElement && isInteractive(child) || Array.from(child.childNodes).some(child => {
        return child instanceof HTMLElement && isInteractive(child);
      });
    });
    // PageHeader.TitleArea is be the first element in the DOM even when it is not visually the first.
    // Motivation behind this rule to make sure context area and leading action (if they exist) are always rendered after the title (a heading tag)
    // so that screen reader users who are navigating via heading menu won't miss these actions.
    process.env.NODE_ENV !== "production" ? warning.warning(hasInteractiveContent && (hasContextArea || hasLeadingAction), 'When PageHeader.ContextArea or PageHeader.LeadingAction is present, we recommended not to include any interactive items in the PageHeader.TitleArea to make sure the focus order is logical.') : void 0;
  }, [children, rootRef]);
  return /*#__PURE__*/jsxRuntime.jsx(RootBaseComponent, {
    as: as,
    ref: rootRef,
    className: clsx.clsx(PageHeader_module.PageHeader, className),
    "data-has-border": hasBorder ? 'true' : undefined,
    sx: sx,
    "aria-label": ariaLabel,
    role: role,
    children: children
  });
});
const ContextAreaBaseComponent = toggleSxComponent.toggleSxComponent('div');
// PageHeader.ContextArea : Only visible on narrow viewports by default to provide user context of where they are at their journey. `hidden` prop available
// to manage their custom visibility but consumers should be careful if they choose to hide this on narrow viewports.
// PageHeader.ContextArea Sub Components: PageHeader.ParentLink, PageHeader.ContextBar, PageHeader.ContextAreaActions
// ---------------------------------------------------------------------
const ContextArea = ({
  children,
  className,
  hidden = hiddenOnRegularAndWide,
  sx: sxProp = defaultSxProp.defaultSxProp
}) => {
  return /*#__PURE__*/jsxRuntime.jsx(ContextAreaBaseComponent, {
    className: clsx.clsx(PageHeader_module.ContextArea, className),
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
ContextArea.displayName = "ContextArea";
// PageHeader.ParentLink : Only visible on narrow viewports by default to let users navigate up in the hierarchy.
const ParentLink = /*#__PURE__*/React__default.default.forwardRef(({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  href,
  'aria-label': ariaLabel,
  as = 'a',
  hidden = hiddenOnRegularAndWide
}, ref) => {
  return /*#__PURE__*/jsxRuntime.jsx(jsxRuntime.Fragment, {
    children: /*#__PURE__*/jsxRuntime.jsxs(Link, {
      ref: ref,
      as: as,
      "aria-label": ariaLabel,
      muted: true,
      className: clsx.clsx(PageHeader_module.ParentLink, className),
      sx: sxProp,
      ...getHiddenDataAttributes(hidden),
      href: href,
      children: [/*#__PURE__*/jsxRuntime.jsx(octiconsReact.ArrowLeftIcon, {}), /*#__PURE__*/jsxRuntime.jsx(Box, {
        children: children
      })]
    })
  });
});
ParentLink.displayName = 'ParentLink';

// ContextBar
// Generic slot for any component above the title region. Use it for custom breadcrumbs and other navigation elements instead of ParentLink.
// ---------------------------------------------------------------------

const ContextBarBaseComponent = toggleSxComponent.toggleSxComponent('div');
const ContextBar = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = hiddenOnRegularAndWide
}) => {
  return /*#__PURE__*/jsxRuntime.jsx(ContextBarBaseComponent, {
    className: clsx.clsx(PageHeader_module.ContextBar, className),
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
ContextBar.displayName = "ContextBar";
const ContextAreaActionsBaseComponent = toggleSxComponent.toggleSxComponent('div');
// ContextAreaActions
// ---------------------------------------------------------------------
const ContextAreaActions = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = hiddenOnRegularAndWide
}) => {
  return /*#__PURE__*/jsxRuntime.jsx(ContextAreaActionsBaseComponent, {
    className: clsx.clsx(PageHeader_module.ContextAreaActions, className),
    ...getHiddenDataAttributes(hidden),
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
ContextAreaActions.displayName = "ContextAreaActions";
// PageHeader.TitleArea: The main title area of the page. Visible on all viewports.
// PageHeader.TitleArea Sub Components: PageHeader.LeadingVisual, PageHeader.Title, PageTitle.TrailingVisual
// ---------------------------------------------------------------------

const TitleAreaBaseComponent = toggleSxComponent.toggleSxComponent('div');
const TitleArea = /*#__PURE__*/React__default.default.forwardRef(({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false,
  variant = 'medium'
}, forwardedRef) => {
  const titleAreaRef = useProvidedRefOrCreate.useProvidedRefOrCreate(forwardedRef);
  const currentVariant = useResponsiveValue.useResponsiveValue(variant, 'medium');
  return /*#__PURE__*/jsxRuntime.jsx(TitleAreaBaseComponent, {
    className: clsx.clsx(PageHeader_module.TitleArea, className),
    ref: titleAreaRef,
    "data-component": "TitleArea",
    "data-size-variant": currentVariant,
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
});
TitleArea.displayName = 'TitleArea';
const LeadingActionBaseComponent = toggleSxComponent.toggleSxComponent('div');
// PageHeader.LeadingAction and PageHeader.TrailingAction should only be visible on regular viewports.
// So they come as hidden on narrow viewports by default and their visibility can be managed by their `hidden` prop.
const LeadingAction = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = hiddenOnNarrow
}) => {
  const style = {};
  // @ts-ignore sx has height attribute
  const {
    height
  } = sxProp;
  if (height) style['--custom-height'] = height;
  return /*#__PURE__*/jsxRuntime.jsx(LeadingActionBaseComponent, {
    className: clsx.clsx(PageHeader_module.LeadingAction, className),
    "data-component": "PH_LeadingAction",
    sx: sxProp,
    style: style,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
LeadingAction.displayName = "LeadingAction";
const BreadcrumbsBaseComponent = toggleSxComponent.toggleSxComponent('div');
// This is reserved for only breadcrumbs.
const Breadcrumbs = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false
}) => {
  return /*#__PURE__*/jsxRuntime.jsx(BreadcrumbsBaseComponent, {
    className: clsx.clsx(PageHeader_module.Breadcrumbs, className),
    "data-component": "PH_Breadcrumbs",
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
Breadcrumbs.displayName = "Breadcrumbs";
const LeadingVisualBaseComponent = toggleSxComponent.toggleSxComponent('div');
// PageHeader.LeadingVisual and PageHeader.TrailingVisual should remain visible on narrow viewports.
const LeadingVisual = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false
}) => {
  const style = {};
  // @ts-ignore sx has height attribute
  const {
    height
  } = sxProp;
  if (height) style['--custom-height'] = height;
  return /*#__PURE__*/jsxRuntime.jsx(LeadingVisualBaseComponent, {
    className: clsx.clsx(PageHeader_module.LeadingVisual, className),
    "data-component": "PH_LeadingVisual",
    sx: sxProp,
    style: style,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
LeadingVisual.displayName = "LeadingVisual";
const Title = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false,
  as = 'h2'
}) => {
  const style = {};
  // @ts-ignore sxProp can have color attribute
  const {
    fontSize,
    lineHeight,
    fontWeight
  } = sxProp;
  if (fontSize) style['--custom-font-size'] = fontSize;
  if (lineHeight) style['--custom-line-height'] = lineHeight;
  if (fontWeight) style['--custom-font-weight'] = fontWeight;
  return /*#__PURE__*/jsxRuntime.jsx(Heading, {
    className: clsx.clsx(PageHeader_module.Title, className),
    "data-component": "PH_Title",
    "data-hidden": hidden,
    as: as,
    style: style,
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
Title.displayName = "Title";
const TrailingVisualBaseComponent = toggleSxComponent.toggleSxComponent('div');
// PageHeader.LeadingVisual and PageHeader.TrailingVisual should remain visible on narrow viewports.
const TrailingVisual = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false
}) => {
  const style = {};
  // @ts-ignore sx has height attribute
  const {
    height
  } = sxProp;
  if (height) style['--custom-height'] = height;
  return /*#__PURE__*/jsxRuntime.jsx(TrailingVisualBaseComponent, {
    className: clsx.clsx(PageHeader_module.TrailingVisual, className),
    "data-component": "PH_TrailingVisual",
    sx: sxProp,
    style: style,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
TrailingVisual.displayName = "TrailingVisual";
const TrailingActionBaseComponent = toggleSxComponent.toggleSxComponent('div');
const TrailingAction = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = hiddenOnNarrow
}) => {
  const style = {};
  // @ts-ignore sx has height attribute
  const {
    height
  } = sxProp;
  if (height) style['--custom-height'] = height;
  return /*#__PURE__*/jsxRuntime.jsx(TrailingActionBaseComponent, {
    className: clsx.clsx(PageHeader_module.TrailingAction, className),
    "data-component": "PH_TrailingAction",
    sx: sxProp,
    style: style,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
TrailingAction.displayName = "TrailingAction";
const ActionsBaseComponent = toggleSxComponent.toggleSxComponent('div');
const Actions = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false
}) => {
  const style = {};
  // @ts-ignore sx has height attribute
  const {
    height
  } = sxProp;
  if (height) style['--custom-height'] = height;
  return /*#__PURE__*/jsxRuntime.jsx(ActionsBaseComponent, {
    className: clsx.clsx(PageHeader_module.Actions, className),
    "data-component": "PH_Actions",
    sx: sxProp,
    style: style,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
Actions.displayName = "Actions";
const DescriptionBaseComponent = toggleSxComponent.toggleSxComponent('div');

// PageHeader.Description: The description area of the header. Visible on all viewports
const Description = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false
}) => {
  return /*#__PURE__*/jsxRuntime.jsx(DescriptionBaseComponent, {
    className: clsx.clsx(PageHeader_module.Description, className),
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
Description.displayName = "Description";
const NavigationBaseComponent = toggleSxComponent.toggleSxComponent('div');

// PageHeader.Navigation: The local navigation area of the header. Visible on all viewports
const Navigation = ({
  children,
  className,
  sx: sxProp = defaultSxProp.defaultSxProp,
  hidden = false,
  as,
  'aria-label': ariaLabel,
  'aria-labelledby': ariaLabelledBy
}) => {
  process.env.NODE_ENV !== "production" ? warning.warning(as === 'nav' && !ariaLabel && !ariaLabelledBy, 'Use `aria-label` or `aria-labelledby` prop to provide an accessible label to the `nav` landmark for assistive technology') : void 0;
  return /*#__PURE__*/jsxRuntime.jsx(NavigationBaseComponent, {
    as: as
    // Render `aria-label` and `aria-labelledby` only on `nav` elements
    ,
    "aria-label": as === 'nav' ? ariaLabel : undefined,
    "aria-labelledby": as === 'nav' ? ariaLabelledBy : undefined,
    className: clsx.clsx(PageHeader_module.Navigation, className),
    "data-component": "PH_Navigation",
    sx: sxProp,
    ...getHiddenDataAttributes(hidden),
    children: children
  });
};
Navigation.displayName = "Navigation";
// Based on getBreakpointDeclarations, this function will return the
// correct data attribute for the given hidden value for CSS modules.
function getHiddenDataAttributes(isHidden) {
  if (useResponsiveValue.isResponsiveValue(isHidden)) {
    const responsiveValue = isHidden;

    // Build media queries with the giving cssProperty and mapFn
    const narrowMediaQuery = 'narrow' in responsiveValue ? {
      'data-hidden-narrow': responsiveValue.narrow || undefined
    } : {};
    const regularMediaQuery = 'regular' in responsiveValue ? {
      'data-hidden-regular': responsiveValue.regular || undefined
    } : {};
    const wideMediaQuery = 'wide' in responsiveValue ? {
      'data-hidden-wide': responsiveValue.wide || undefined
    } : {};

    // check if all values are the same - this is not a recommended practice but we still should check for it
    if (getBreakpointDeclarations.areAllValuesTheSame(responsiveValue)) {
      // if all the values are the same, we can just use one of the value to determine the CSS property's value
      return {
        'data-hidden-all': responsiveValue.narrow || undefined
      };
      // check if regular and wide have the same value, if so we can just return the narrow and regular media queries
    } else if (getBreakpointDeclarations.haveRegularAndWideSameValue(responsiveValue)) {
      return {
        ...narrowMediaQuery,
        ...regularMediaQuery
      };
    } else {
      return {
        ...narrowMediaQuery,
        ...regularMediaQuery,
        ...wideMediaQuery
      };
    }
  } else {
    // If the given value is not a responsive value
    return {
      'data-hidden-all': isHidden || undefined
    };
  }
}
const PageHeader = Object.assign(Root, {
  ContextArea,
  ParentLink,
  ContextBar,
  TitleArea,
  ContextAreaActions,
  LeadingAction,
  Breadcrumbs,
  LeadingVisual,
  Title,
  TrailingVisual,
  TrailingAction,
  Actions,
  Description,
  Navigation
});
PageHeader.displayName = 'PageHeader';

exports.PageHeader = PageHeader;
